#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "const.h"

char *prog_name;

void version()
{
  fprintf(stdout,"%s, %s - %s - Version %s\n",prog_name,Auteur,Date,Version);  
}

void display_stat(int Stat[260], Options opt)
{
  int i;
  int j=0;

  fprintf(stdout,"Numro\tNombre\n");
  for (i=0;i<=255;i++)
    {
      j++;
      fprintf(stdout,"%3d %7d | ",i,Stat[i]);
      if (j==opt.nb_cols)
	{
	  j=0;
	  fprintf(stdout,"\n");
	}
    }
}

void display_ascii()
{
  int i;

  fprintf(stdout,"Numro\tType\tDescription\n");
  for (i=0;i<=MAX_RANGE;i++)
    if (ascii[i].type==TEXTE)
      fprintf(stdout,"%3d\t%s\t%s\n",i,"Texte",ascii[i].desc);
    else
      if (ascii[i].type==BINAIRE)
	fprintf(stdout,"%3d\t%s\t%s\n",i,"Binaire",ascii[i].desc);
}


void display_results(char *filename, int val, int Stat[260], Options opt)
{
   if (opt.aff_ext == SELECTED)
    {
      switch (val)
	{
	case BINAIRE:
	  fprintf(stdout,"%s est de type binaire",filename);
	  break;
    
	case TEXTE:
	  fprintf(stdout,"%s est de type texte",filename);
	  break; 
    
	case INCONNU:
	  fprintf(stdout,"%s est de type inconnu",filename);
	  break;
    
	case ERREUR:
	  fprintf(stdout,"%s a provoqu une erreur\n",filename);
	  break;
	}
      if (val!=ERREUR)
	fprintf(stdout,"\n\tTaille du fichier : %d\n\tNombre de caractres de type binaire :\t%d\n\tNombre de caractres de type texte :\t%d\n\tNombre de caractres d'aucun type :\t%d\n", Stat[256],Stat[257],Stat[258],Stat[259]);
    }
   else
     fprintf(stdout,"%d\n",val);
  if (opt.aff_check == SELECTED )
    display_stat(Stat, opt);
}


int usage(int status)
{
 if (status != 0)
    {
      fprintf (stderr, 
	       "Pour en savoir davantage, faites: `%s --help'\n",prog_name);
    }
  else
    {
      version();
      fprintf(stdout,"Donne le type d'un fichier : Texte / Binaire\n");
      fprintf(stdout,"\nUsage : \n\
  %s [options] FICHIER\n",prog_name);
      fprintf(stdout,"\nOptions :\n\ 
  -h, --help\tCette aide.\n\
  -v, --version\tAffiche la version du programme.\n\
  -d, --display\tAffiche la table de caractres ascii (8 bits) utilise.\n\
  -c, --check=NBCOLS\tAffiche les statistiques du fichier, pour vrifications,\n\
                    \tSur NBCOLS colonnes.
  -e, --extended\tAffiche plus d'information sur le rsultat.\n");
      fprintf(stdout,"\nSorties :\n\
  %d Fichier binaire\n\
  %d Fichier texte\n\
  %d Fichier inconnu\n\
  %d Erreur de lecture\n",BINAIRE,TEXTE,INCONNU,ERREUR);
    }
  return TRUE;
}

/*
  Stat : 
  0 - 255 le nombre de caractres correspondant au caractre #i
  256 La taille du fichier
  257 Le nombre de caractres de type binaire
  258 Le nombre de caractres de type texte
  259 Le nombre de caractres d'aucun type (>MAX_RANGE <=255)
*/

int istext(char *filename, int Stat[260])
{
  FILE *fp;
  int filedes;
  int c;
  int b;
  int t;
  struct stat *buf;

  for(c=0;c<=259;c++)
    Stat[c] = 0;

  if ( (fp=fopen(filename,"r"))!=NULL)
    {
      buf = (struct stat*) malloc (sizeof(struct stat));
      filedes=fileno(fp);
      fstat(filedes,buf);
      if (S_ISREG(buf->st_mode))
	{
	  // Statistiques sur le fichier.
	  while (!feof(fp))
	    {
	      c = fgetc(fp);
	      Stat[c]++;
	      Stat[256]++;
	    }

	  // Connaissance du type du fichier
	  c=0;
	  b=0;
	  t=0;
	  while (c<=MAX_RANGE)
	    {
	      if (ascii[c].type==BINAIRE && Stat[c]!=0)
		b += Stat[c];
	      else
		t += Stat[c];

	      c++;
	    }
	  while (c<=255)
	    {
	      Stat[259] += Stat[c];
	      c++;
	    }
	  Stat[257]=b;
	  Stat[258]=t;
	  if (b!=0)
	    return BINAIRE;
	  else 
	    return TEXTE;
	}
      else
	return ERREUR;
      free(buf);
    }
  else
    return ERREUR;
}



int main (int argc, char ** argv)
{
  int c = 0;
  int help = FALSE;
  int Stat[260];
  int val = 0;
  char *filename;
  Options opt;

  opt.aff_ext = NONE;
  opt.aff_check = NONE;

  prog_name = (char*) malloc (sizeof(char)*256);
  prog_name = argv[0];

  while ((c = getopt_long (argc, argv, "vhdce", long_options, NULL)) != -1)
    {
      switch (c)
	{
	case 0:			/* Long option. */
	  break;
	  
	case 'v':
	  version();
	  break;
	 
	case 'h':
	  help = TRUE; 
	  break;

	case 'd':
	  display_ascii();
	  break;

	case 'c':
	  if (optarg)
	    {
	      if (sscanf(optarg,"%d",&opt.nb_cols)<1)
		opt.nb_cols = 1;
	    }
	  else
	    opt.nb_cols = 1;
	  if (opt.nb_cols<1 || opt.nb_cols>255)
	    opt.nb_cols = 1;
	  
	  opt.aff_check = SELECTED;
	  break;

	case 'e':
	  opt.aff_ext = SELECTED;
	  break;

	default:
	  usage(1);
	}
    }

 // les arguments qui ne sont pas dans ceux dfinis :
  if (optind < argc)
    {
      while (optind < argc)
	{
	  filename = (char*) malloc (sizeof(char)*strlen(argv[optind])+1);
	  strcpy(filename,argv[optind]);
	  optind++;
	  val = istext(filename, Stat);
	  display_results(filename, val, Stat, opt);
	  free(filename);
	}
      printf ("\n");
    }
  else
    {
      if (help == TRUE)
	usage(0);
    }
  

  free(prog_name);
  return TRUE;
}
